@@ -1,2 +1,2 @@
 from utaupy import (backup, convert, label, otoini, reaper, reclist, shiro,
-                    svp, table, ust, utauplugin)
+                    svp, table, ust, utau, utauplugin)
